using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace CheeseBreakout
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public class BreadAndCheeseComponent : Microsoft.Xna.Framework.DrawableGameComponent
    {
        SpriteBatch spriteBatch;

        Texture2D cheeseTexture;
        Rectangle cheeseRectangle;

        Texture2D breadTexture;
        Rectangle breadRectangle;

        float cheeseX = 0;
        float cheeseY = 0;

        float breadX;
        float breadY;

        float xSpeed = 1;
        float ySpeed = 1;

        float padXSpeed = 10;
        float padYSpeed = 10;

        float cheesesAcrossScreen = 11;
        float breadAccrossScreen = 6;

        public BreadAndCheeseComponent(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(Game.GraphicsDevice);

            cheeseTexture = Game.Content.Load<Texture2D>("cheese");
            cheeseRectangle.Width = (int) ((Game.GraphicsDevice.Viewport.Width / cheesesAcrossScreen) +0.5f);
            cheeseRectangle.Height = cheeseRectangle.Width; 

            breadTexture = Game.Content.Load<Texture2D>("bread");
            breadRectangle.Width = (int) ((Game.GraphicsDevice.Viewport.Width / breadAccrossScreen) + 0.5f);
            breadRectangle.Height = breadRectangle.Width / 3;
            this.DrawOrder = 1;
            base.Initialize();
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                Game.Exit();

            cheeseX += xSpeed;

            if ((cheeseX <= 0) ||
                (cheeseX + cheeseRectangle.Width > Game.GraphicsDevice.Viewport.Width))
            {
                xSpeed *= -1;
            }

            cheeseY += ySpeed;

            if ((cheeseY <= 0) ||
                (cheeseY + cheeseRectangle.Height > Game.GraphicsDevice.Viewport.Height))
            {
                ySpeed *= -1;
            }

            GamePadState padState = GamePad.GetState(PlayerIndex.One);

            if (padState.IsConnected)
            {
                breadX += padState.ThumbSticks.Left.X * padXSpeed;
                breadY -= padState.ThumbSticks.Left.Y * padYSpeed;
            }
            
            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin();
            cheeseRectangle.X = (int)(cheeseX + 0.5f);
            cheeseRectangle.Y = (int)(cheeseY + 0.5f);
            spriteBatch.Draw(cheeseTexture,cheeseRectangle,Color.White);

            breadRectangle.X = (int)(breadX + 0.5f);
            breadRectangle.Y = (int)(breadY + 0.5f);
            spriteBatch.Draw(breadTexture, breadRectangle,Color.White);

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}