using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SillyText
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class SillyTextGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        /// <summary>
        /// Font used to draw all the text in our messages.
        /// </summary>
        SpriteFont font;

        Color[] titleColors = new Color[] { Color.Red, Color.Blue, Color.Green, Color.DarkOrange, Color.Purple, Color.Pink, Color.LightBlue, Color.DarkBlue, Color.Yellow, Color.Blue, Color.Green, Color.Magenta, Color.Purple };

        Color[] sides = new Color[] { Color.Black };

        Color[] shadows = new Color[] { new Color(120, 120, 120, 255) };

        SillyChar testChar;

        SillyLine testLine;

        public SillyTextGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Load the text font
            font = Content.Load<SpriteFont>("SillyFont");
            //testChar = new SillyChar("test", font, Color.White, Color.Gray, new Color(0x0,0x0,0x0,0x40),new Vector2(200,200));
            //testChar.StartWander();
            //testChar.StartRotate(-0.03f);

            testLine = new SillyLine("VERYSILLYGAMES", new Vector2(30, 300), titleColors, sides, shadows, font);

            //testChar.StartRotate(0.01f);
            //testChar.StartRotate(0.02f);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        GamePadState oldPad = GamePad.GetState(PlayerIndex.One);

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            GamePadState pad = GamePad.GetState(PlayerIndex.One);

            testLine.LightAngle = (float) (Math.Atan2(-pad.ThumbSticks.Left.Y, pad.ThumbSticks.Left.X));// / ((2*Math.PI)/360));
            testLine.ShadowDistance = (float)Math.Sqrt(
                (pad.ThumbSticks.Left.X * pad.ThumbSticks.Left.X) +
                (pad.ThumbSticks.Left.Y * pad.ThumbSticks.Left.Y) ) * 100;

            if (pad.Buttons.LeftShoulder == ButtonState.Pressed && oldPad.Buttons.LeftShoulder == ButtonState.Released)
            {
                testLine.WanderState = SillyChar.WanderStates.wandering;
            }

            if (pad.Buttons.RightShoulder == ButtonState.Pressed && oldPad.Buttons.RightShoulder == ButtonState.Released)
            {
                testLine.StartTwizzle();
            }

            if (pad.Buttons.A == ButtonState.Pressed && oldPad.Buttons.A == ButtonState.Released)
            {
                testLine.Straighten(100,200);
            }

            if (pad.Buttons.B == ButtonState.Pressed && oldPad.Buttons.B == ButtonState.Released)
            {
                testLine.Straighten(100,200);
            }

            if (pad.Buttons.X == ButtonState.Pressed && oldPad.Buttons.X == ButtonState.Released)
            {
                testLine.returnToPosition(100,200);
            }

            if (pad.Buttons.Y == ButtonState.Pressed && oldPad.Buttons.Y == ButtonState.Released)
            {
                testLine.WanderState = SillyChar.WanderStates.returning;
            }
            //testChar.Update();
            testLine.Update();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.White);

            spriteBatch.Begin();

            //testChar.Draw(spriteBatch);
            testLine.Draw(spriteBatch);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
