using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CheeseBreakout
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class CheeseGame : Microsoft.Xna.Framework.DrawableGameComponent, IGetStatusText
    {
        private int scoreValue;

        private int livesValue;

        private SpriteFont font;

        public string GetStatus()
        {
            return "Score: " + scoreValue.ToString() + "  Lives: " + livesValue;
        }

        public SpriteBatch SpriteBatchDraw
        {
            get
            {
                return spriteBatch;
            }
        }

        public SpriteFont FontDraw
        {
            get
            {
                return font;
            }

        }

        public CheeseGame(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        SpriteBatch spriteBatch;
        ContentManager content;

        int cheeseX = 0;
        int cheeseY = 0;

        int breadX;
        int breadY;

        int padXSpeed = 10;
        int padYSpeed = 10;

        int cheeseXSpeed = 1;
        int cheeseYSpeed = 1;



        KeyboardState oldKeyboard = Keyboard.GetState();

        float xSpeed, ySpeed;

        float xminspeed = 1;
        float xmaxSpeed = 20;
        float xspeedStep = 0.7f;

        float yminspeed = 1;
        float ymaxSpeed = 20;
        float yspeedStep = 0.7f;


        /// <summary>
        /// The texture which represents our cheese
        /// </summary>
        Texture2D cheeseTexture;

        /// <summary>
        /// The texture which represents our bread
        /// </summary>
        Texture2D breadTexture;

        Texture2D tomatoTexture;
        Texture2D pepperTexture;

        Rectangle cheeseRect;
        Rectangle breadRect;
        Rectangle tomatoRect;
        Rectangle pepperRect;

        int startTomatoes = 5;
        int startPeppers = 2;

        private void gameStart()
        {
            foreach (gameTarget t in targets)
            {
                t.TargetType = targetTypes.invisible;
            }

            for (int i = 0; i < startTomatoes; i++)
            {
                placeTomato();
            }

            for (int i = 0; i < startPeppers; i++)
            {
                placePepper();
            }

            cheeseRect.X = (this.Game.Window.ClientBounds.Width - cheeseRect.Width) / 2;
            cheeseRect.Y = (this.Game.Window.ClientBounds.Height - cheeseRect.Height) / 2;

            breadRect.X = (this.Game.Window.ClientBounds.Width - breadRect.Width) / 2;
            breadRect.Y = this.Game.Window.ClientBounds.Height - breadRect.Height;

            scoreValue = 0;
            livesValue = 3;
        }

        private void scaleSprites()
        {
            cheeseRect.Width = this.Game.Window.ClientBounds.Width / 15;
            cheeseRect.Height = this.Game.Window.ClientBounds.Width / 15;
            breadRect.Width = this.Game.Window.ClientBounds.Width / 5;
            breadRect.Height = this.Game.Window.ClientBounds.Width / 20;
            tomatoRect.Width = this.Game.Window.ClientBounds.Width / 15;
            tomatoRect.Height = this.Game.Window.ClientBounds.Width / 15;
            pepperRect.Width = this.Game.Window.ClientBounds.Width / 15;
            pepperRect.Height = this.Game.Window.ClientBounds.Width / 15;
        }

        Sound sound;

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            MainGame parent = this.Game as MainGame;
            content = parent.content;
            this.DrawOrder = 1;
            sound = new Sound();
            sound.Initialize();
            base.Initialize();

        }

        protected override void LoadContent()
        {
            cheeseTexture = content.Load<Texture2D>("cheese");
            breadTexture = content.Load<Texture2D>(@"bread");
            pepperTexture = content.Load<Texture2D>(@"pepper");
            tomatoTexture = content.Load<Texture2D>(@"tomato");

            cheeseRect = new Rectangle(0, 0, cheeseTexture.Width, cheeseTexture.Height);
            breadRect = new Rectangle(0, 0, breadTexture.Width, breadTexture.Height);
            tomatoRect = new Rectangle(0, 0, tomatoTexture.Width, tomatoTexture.Height);
            pepperRect = new Rectangle(0, 0, pepperTexture.Width, pepperTexture.Height);

            font = content.Load<SpriteFont>("MessageFont");

            scaleSprites();

            spriteBatch = new SpriteBatch(this.GraphicsDevice);
            base.LoadContent();
        }

        private bool intersectsWith(Rectangle r1, Rectangle r2)
        {
            return !(r2.Left > r1.Right
                    || r2.Right < r1.Left
                    || r2.Top > r1.Bottom
                    || r2.Bottom < r1.Top
                    );
        }

        private enum targetTypes
        {
            tomato,
            pepper,
            invisible
        }

        private class gameTarget
        {
            public targetTypes TargetType;
            public Rectangle SourceRect;
            public Rectangle PosRect;
            public Texture2D TagetTexture;
            public Vector2 Origin;
            public float Rotation;
            public float RotateSpeed;
            public static double RotateFactor = 0.2;

            private static Random rand = new Random();

            public gameTarget(
                targetTypes inTargetType, Rectangle inPosRect, Texture2D inTagetTexture)
            {
                TargetType = inTargetType;
                PosRect = inPosRect;
                TagetTexture = inTagetTexture;
                Origin.X = TagetTexture.Width / 2;
                Origin.Y = TagetTexture.Height / 2;
                RotateSpeed = (float)((RotateFactor / 2) - rand.NextDouble() * RotateFactor);
                SourceRect = new Rectangle(0, 0, TagetTexture.Width, TagetTexture.Height);
            }

            public void Draw(SpriteBatch spriteBatch)
            {
                if (TargetType != targetTypes.invisible)
                {
                    spriteBatch.Draw(
                        TagetTexture,
                        PosRect,
                        SourceRect,
                        Color.White,
                        Rotation,
                        Origin,
                        SpriteEffects.None,
                        0);
                }
            }

            public void Update()
            {
                Rotation += RotateSpeed;
            }
        }

        private List<gameTarget> targets = new List<gameTarget>();
        private List<gameTarget> peppers = new List<gameTarget>();

        private int maxNoOfTomatoes = 50;
        private int maxNoOfPeppers = 10;

        private int noOfActiveTomatoes;
        private int noOfActivePeppers;

        private System.Random rand = new Random(1);

        private void drawTargets()
        {
            foreach (gameTarget t in targets)
            {
                t.Draw(spriteBatch);
            }
        }

        private void updateTargets()
        {
            foreach (gameTarget t in targets)
            {
                t.Update();
            }
        }

        private void setupTargets()
        {
        }

        private bool hitsTarget(Rectangle rect)
        {
            foreach (gameTarget t in targets)
            {
                if (t.TargetType == targetTypes.invisible)
                {
                    continue;
                }
                if (intersectsWith(t.PosRect, rect))
                {
                    return true;
                }
            }
            return false;
        }

        private bool randomPlaceTarget(gameTarget t)
        {
            int xLimit = this.Game.Window.ClientBounds.Width - t.PosRect.Width;
            int yLimit = this.Game.Window.ClientBounds.Height - t.PosRect.Height;
            int tries;

            for (tries = 0; tries < 10; tries++)
            {
                t.PosRect.X = rand.Next(xLimit);
                t.PosRect.Y = rand.Next(yLimit);
                if (!hitsTarget(t.PosRect))
                {
                    return true;
                }
            }
            return false;
        }

        private bool placeTarget(targetTypes type, Texture2D texture, Rectangle rect)
        {
            gameTarget result = null;

            foreach (gameTarget t in targets)
            {
                if (t.TargetType == targetTypes.invisible)
                {
                    result = t;
                    break;
                }
            }

            if (result == null)
            {
                result = new gameTarget(targetTypes.invisible, rect, texture);
                targets.Add(result);
            }

            if (!randomPlaceTarget(result))
            {
                return false;
            }

            result.TargetType = type;
            return true;
        }

        private bool placeTomato()
        {
            noOfActiveTomatoes++;
            return placeTarget(targetTypes.tomato, tomatoTexture, tomatoRect);
        }

        private bool placePepper()
        {
            noOfActivePeppers++;
            return placeTarget(targetTypes.pepper, pepperTexture, pepperRect);
        }

        private void updatePositions()
        {
            cheeseRect.X += cheeseXSpeed;

            if (cheeseRect.X <= 0)
            {
                cheeseXSpeed = Math.Abs(cheeseXSpeed);
            }
            if (cheeseRect.X + cheeseRect.Width > this.Game.Window.ClientBounds.Width)
            {
                cheeseXSpeed = -Math.Abs(cheeseXSpeed);
            }

            cheeseRect.Y += cheeseYSpeed;

            if (cheeseRect.Y <= 0)
            {
                cheeseYSpeed = Math.Abs(cheeseYSpeed);
            }

            if (cheeseRect.Y + cheeseRect.Height > this.Game.Window.ClientBounds.Height)
            {
                cheeseYSpeed = -Math.Abs(cheeseYSpeed);
            }

            GamePadState padState = GamePad.GetState(PlayerIndex.One);

            if (padState.IsConnected)
            {
                if (padState.Buttons.Start == ButtonState.Pressed)
                {
                    gameStart();
                }
                float padX = (padState.ThumbSticks.Left.X * padXSpeed);
                float padY = (padState.ThumbSticks.Left.Y * padYSpeed);
                breadRect.X += (int)padX;
                breadRect.Y -= (int)padY;
                if (intersectsWith(cheeseRect, breadRect))
                {
                    cheeseXSpeed = -(int)padX;
                    cheeseYSpeed = (int)padY;
                }
            }


            updateTargets();
        }

        private void checkScore()
        {
            foreach (gameTarget t in targets)
            {
                if (t.TargetType == targetTypes.invisible)
                {
                    continue;
                }
                if (intersectsWith(t.PosRect, cheeseRect))
                {
                    if (t.TargetType == targetTypes.tomato)
                    {
                        Sound.Play(Sound.Sounds.burp);
                        scoreValue += 10;
                        randomPlaceTarget(t);
                    }
                    if (t.TargetType == targetTypes.pepper)
                    {
                        Sound.Play(Sound.Sounds.implosion);
                        livesValue -= 1;
                        randomPlaceTarget(t);
                    }
                }
            }
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                Game.Exit();

            updatePositions();

            checkScore();

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            spriteBatch.Begin();

            drawTargets();
            spriteBatch.Draw(cheeseTexture, cheeseRect, Color.White);
            spriteBatch.Draw(breadTexture, breadRect, Color.White);

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
