using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace CheeseBreakout
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class Sound
    {

        public enum Sounds
        {
            burp,
            implosion
        }

        private static string[] cueNames = new string[]
        {
            "Burp",
            "implosion"
        };

        /// <summary>
        /// Plays a sound
        /// </summary>
        /// <param name="sound">Which sound to play</param>
        /// <returns>XACT cue to be used if you want to stop this particular looped sound. Can be ignored for one shot sounds</returns>
        public static Cue Play(Sounds sound)
        {
            Cue returnValue = sound_bank.GetCue(cueNames[(int)sound]);
            returnValue.Play();
            return returnValue;
        }

        /// <summary>
        /// Stops a previously playing cue
        /// </summary>
        /// <param name="cue">The cue to stop that you got returned from Play(sound)</param>
        public static void Stop(Cue cue)
        {
            cue.Stop(AudioStopOptions.Immediate);
        }


        static AudioEngine audio_engine;
        static WaveBank wave_bank;
        static SoundBank sound_bank;

        public void Initialize()
        {
            audio_engine = new AudioEngine("Content\\Gm.xgs");
            wave_bank = new WaveBank(audio_engine, "Content\\Wave Bank.xwb");
            sound_bank = new SoundBank(audio_engine, "Content\\Sound Bank.xsb");
        }
    }
}
