using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace CheeseBreakout
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class Rocket : Microsoft.Xna.Framework.DrawableGameComponent
    {
        public Rocket(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }

        //Position of the model in world space, and rotation
        Vector3 modelposition = Vector3.Zero;
        float modelrotation = 0.0f;

        //Position of the Camera in world space, for our view matrix
        Vector3 camposition = new Vector3(0.0f, 50.0f, 3000.0f);

        //Aspect ratio to use for the projection matrix
        float aspectRatio = 640.0f / 480.0f;

        //Velocity of the model, applied each frame to the model's position
        Vector3 modelvelocity = Vector3.Zero;

        Model rocket;

        ContentManager content;

        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            MainGame parent = this.Game as MainGame;
            content = parent.content;
            this.DrawOrder = 2;
            base.Initialize();
        }

        protected override void LoadContent()
        {
            rocket = content.Load<Model>(@"models\p1_rocket");
            base.LoadContent();
        }

        protected void UpdateInput()
        {
            //get the gamepad state
            GamePadState current_state = GamePad.GetState(PlayerIndex.One);
            if (current_state.IsConnected)
            {
                //rotate the model using the left thumbstick, scale it down
                modelrotation -= current_state.ThumbSticks.Left.X * 0.10f;

                //create some velocity if the right trigger is down
                Vector3 modelvelocity_add = Vector3.Zero;

                //find out what direction we should be thrusting, using rotation
                modelvelocity_add.X = -(float)Math.Sin(modelrotation);
                modelvelocity_add.Z = (float)Math.Cos(modelrotation);

                //now scale our direction by how hard the trigger is down
                modelvelocity_add *= current_state.Triggers.Right;

                //finally, add this vector to our velocity.
                modelvelocity += modelvelocity_add;

                GamePad.SetVibration(PlayerIndex.One, current_state.Triggers.Right,
                    current_state.Triggers.Right);

                //in case you get lost, press A to warp back to the center
                if (current_state.Buttons.A == ButtonState.Pressed)
                {
                    modelposition = Vector3.Zero;
                    modelvelocity = Vector3.Zero;
                    modelrotation = 0.0f;
                }
            }
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            //add velocity to current position
            modelposition += modelvelocity;

            //bleed off velocity over time
            modelvelocity *= 0.95f;

            UpdateInput();

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            //Draw the model, a model can have multiple meshes, so loop
            foreach (ModelMesh mesh in rocket.Meshes)
            {
                //This is where the mesh orientation is set, as well as our camera and projection
                foreach (BasicEffect effect in mesh.Effects)
                {
                    effect.EnableDefaultLighting();
                    effect.World = Matrix.CreateRotationX(modelrotation) * mesh.ParentBone.Transform
                        * Matrix.CreateTranslation(modelposition);
                    effect.View = Matrix.CreateLookAt(camposition, Vector3.Zero, Vector3.Up);
                    effect.Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f),
                        aspectRatio, 1.0f, 10000.0f);
                }
                //Draw the mesh, will use the effects set above.
                mesh.Draw();
            }
            base.Draw(gameTime);
        }
    }
}
