using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Xml;

namespace SillyText
{
    public delegate void RSSUpdated(List<MessageItem> messages);

    public interface IMessageUpdate
    {
        void ShutDown();
        void RequestUpdate();
        RSSUpdated Updated
        {
            get;
            set;
        }
    }


    public class JustMessages : IMessageUpdate 
    {

        public List<MessageItem> MessageLoadBuffer = new List<MessageItem>();

        #region IMessageUpdate Members

        public void ShutDown()
        {
        }

        public void RequestUpdate()
        {
            if (Updated != null)
            {
                Updated(MessageLoadBuffer);
            }

        }

        private RSSUpdated updateReceivedValue;

        public RSSUpdated Updated
        {
            get { return updateReceivedValue; }
            set { updateReceivedValue = value; }
        }

        #endregion

        public JustMessages()
        {
            MessageLoadBuffer.Add(
                new MessageItem(
                    "Welcome",
                    "<p>Welcome to DevDays Academic</P>",
                    DateTime.Now.ToLongTimeString(),
                    FeedDescription.NullFeed,
                    1)
            );

            MessageLoadBuffer.Add(
                new MessageItem(
                    "XNA",
@"<p>These messages are brought to you by the wonder of XNA</P>
<p>You can download the program (which also reads RSS feeds) from www.verysillygames.com</P>",
                    DateTime.Now.ToLongTimeString(),
                    FeedDescription.NullFeed,
                    1)
            );

            MessageLoadBuffer.Add(
                new MessageItem(
                    "Rob Miles",
@"<p>Your host for this session is Rob Miles</P>
<p>Find out what Rob has been doing (if you really want to know) at www.robmiles.com</P>",
                    DateTime.Now.ToLongTimeString(),
                    FeedDescription.NullFeed,
                    1)
            );

            MessageLoadBuffer.Add(
                new MessageItem(
                    "Buy My Book",
@"<p>Don't forget to buy my book</P>
<p>Microsoft XNA Game Studio 2.0: Learn Programming Now!</P>
<p>Buy it, and know that from now on a tiny bit of your money is now mine...</P>",
                    DateTime.Now.ToLongTimeString(),
                    FeedDescription.NullFeed,
                    1)
            );
        }
    }

    public class FeedDescription
    {
        public string URL;
        public string Name;
        public string Username;
        public string Password;
        public string Domain;

        public TimeSpan ItemAge;
        public int MaxNoOfMessages;

        public FeedDescription( 
            string inURL, 
            string inName,
            TimeSpan inItemAge, 
            int inMaxNoOfMessages ) : this (inURL,inName,inItemAge,inMaxNoOfMessages,"","","")
        {
        }
        
        public FeedDescription(
            string inURL,
            string inName,
            TimeSpan inItemAge,
            int inMaxNoOfMessages,
            string inUsername,
            string inPassword,
            string inDomain)
        {
            URL = inURL;
            Name = inName;
            ItemAge = inItemAge;
            MaxNoOfMessages = inMaxNoOfMessages;
            Username = inUsername;
            Password = inPassword;
            Domain = inDomain;
        }

        public override string ToString()
        {
            return "URL : " + URL + " TimeSpan " + ItemAge.ToString() ;
        }

        public static FeedDescription NullFeed =
            new FeedDescription(
                "Null URK", "Null", new TimeSpan(99, 99, 99), 1);

    }

    public class MessageItem
    {

        private int number;

        public int Number
        {
            get
            {
                return number;
            }
        }

        private string title;

        public string Title
        {
            get { return title; }
        }

        private string bodyString;

        public string BodyString
        {
            get
            {
                return bodyString;
            }
        }

        private FeedDescription feed;

        public FeedDescription Feed
        {
            get
            {
                return feed;
            }
        }

        private string pubDate;

        public string PubDate
        {
            get { return pubDate; }
        }

        private object tag = null;

        public object Tag
        {
            get
            {
                return tag;
            }
            set
            {
                tag = value;
            }
        }

        static string[] badBits = new string[] {
            "<div>??</div>",
            "<b>Body:</b>",
            "<div><b>Expires:</b> ../../....</div>"
        };


        public MessageItem(string inTitle, string inBodyString, string inPubDate, FeedDescription  inFeed, int inNumber)
        {
            title = inTitle;

            feed = inFeed;

            foreach (string search in badBits)
            {
                inBodyString = Regex.Replace(inBodyString, search, "");
            }

            inBodyString = Regex.Replace(inBodyString, "\\?\\?", " ");

            bodyString = inBodyString;
            pubDate = inPubDate;
            number = inNumber;
        }

        public override string ToString()
        {
            StringBuilder reply = new StringBuilder();
            reply.Append("No: ");
            reply.Append(number.ToString());
            reply.Append(" Title: ");
            reply.Append(title);
            reply.Append(" Pubdate: ");
            reply.AppendLine(pubDate);
            reply.Append(" Body: ");
            reply.AppendLine(bodyString);

            return reply.ToString();
        }
    }

    public class Messages : IMessageUpdate
    {
        public string title;
        public string bodyText;
        public string date;

        private List<MessageItem> messageLoadBuffer = new List<MessageItem>();

        private RSSUpdated updateReceivedValue;

        public RSSUpdated Updated
        {
            get { return updateReceivedValue; }
            set { updateReceivedValue = value; }
        }

        #region Message Reader

        private StringBuilder readWebPage(
            string url,
            string username,
            string password,
            string domain)
        {
            // used to build entire input
            StringBuilder sb = new StringBuilder();

            // used on each read operation
            byte[] buf = new byte[8192];

            try
            {
                // prepare the web page we will be asking for
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);

                if (username.Length > 0)
                {
                    request.Proxy = null;
                    NetworkCredential credential = new NetworkCredential(username, password, domain);
                    request.Credentials = credential;
                }

                // execute the request
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();

                while (debugOutput.TriggerActive)
                {
                    System.Threading.Thread.Sleep(1000);
                }

                // we will read data via the response stream
                Stream resStream = response.GetResponseStream();

                string tempString = null;
                int count = 0;

                do
                {
                    // fill the buffer with data
                    count = resStream.Read(buf, 0, buf.Length);

                    // make sure we read some data
                    if (count != 0)
                    {
                        // translate from bytes to ASCII text
                        tempString = Encoding.ASCII.GetString(buf, 0, count);

                        // continue building the string
                        sb.Append(tempString);
                    }
                }
                while (count > 0); // any more data to read?

                return sb;
            }
            catch (System.Net.WebException w)
            {
                debugOutput.PutMessageLine(w.Message);
                return null;
            }
            catch (System.Net.ProtocolViolationException p)
            {
                debugOutput.PutMessageLine(p.Message);
                return null;
            }
            catch (Exception e)
            {
                debugOutput.PutMessageLine(e.Message);
                return null;
            }
        }

        private XmlDocument getRSSDoc(
            string contentURL,
            string username,
            string password,
            string domain)
        {
            XmlDocument doc = new XmlDocument();

            string text;

            try
            {
                StringBuilder sb = readWebPage(contentURL, username, password, domain);

                // for some reason the university web feed generater
                // puts a bunch of ? characters before the start of the
                // xml tag, which upsets the document reader
                while (sb[0] == '?')
                {
                    sb.Remove(0, 1);
                }

                text = sb.ToString();
                doc.LoadXml(text);
            }
            catch
            {
                debugOutput.PutMessageLine("XML document load failed " + contentURL);
                return null;
            }
            debugOutput.PutMessageLine("XML document load OK " + contentURL);
            return doc;
        }

        #region Date and Time Decode

        public struct W3CDateTime
        {
            private DateTime dtime;
            private TimeSpan ofs;

            public W3CDateTime(DateTime dt, TimeSpan off)
            {
                ofs = off;
                dtime = dt;
            }

            public DateTime UtcTime
            {
                get { return dtime; }
            }

            public DateTime DateTime
            {
                get { return dtime + ofs; }
            }

            public TimeSpan UtcOffset
            {
                get { return ofs; }
            }

            static public W3CDateTime Parse(string s)
            {
                const string Rfc822DateFormat =
                  @"^((Mon|Tue|Wed|Thu|Fri|Sat|Sun), *)?(?<day>\d\d?) +" +
                  @"(?<month>Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) +" +
                  @"(?<year>\d\d(\d\d)?) +" +
                  @"(?<hour>\d\d):(?<min>\d\d)(:(?<sec>\d\d))? +" +
                  @"(?<ofs>([+\-]?\d\d\d\d)|UT|GMT|EST|EDT|CST|CDT|MST|MDT|PST|PDT)$";
                const string W3CDateFormat =
                  @"^(?<year>\d\d\d\d)" +
                  @"(-(?<month>\d\d)(-(?<day>\d\d)(T(?<hour>\d\d):(?<min>\d\d)(:(?<sec>\d\d)(?<ms>\.\d+)?)?" +
                  @"(?<ofs>(Z|[+\-]\d\d:\d\d)))?)?)?$";

                string combinedFormat = string.Format(
                  @"(?<rfc822>{0})|(?<w3c>{1})", Rfc822DateFormat, W3CDateFormat);

                // try to parse it
                Regex reDate = new Regex(combinedFormat);
                Match m = reDate.Match(s);
                if (!m.Success)
                {
                    // Didn't match either expression. Throw an exception.
                    throw new FormatException("String is not a valid date time stamp.");
                }
                try
                {
                    bool isRfc822 = m.Groups["rfc822"].Success;
                    int year = int.Parse(m.Groups["year"].Value);
                    // handle 2-digit and 3-digit years
                    if (year < 1000)
                    {
                        if (year < 50) year = year + 2000; else year = year + 1999;
                    }

                    int month;
                    if (isRfc822)
                        month = ParseRfc822Month(m.Groups["month"].Value);
                    else
                        month = (m.Groups["month"].Success) ? int.Parse(m.Groups["month"].Value) : 1;

                    int day = m.Groups["day"].Success ? int.Parse(m.Groups["day"].Value) : 1;
                    int hour = m.Groups["hour"].Success ? int.Parse(m.Groups["hour"].Value) : 0;
                    int min = m.Groups["min"].Success ? int.Parse(m.Groups["min"].Value) : 0;
                    int sec = m.Groups["sec"].Success ? int.Parse(m.Groups["sec"].Value) : 0;
                    int ms = m.Groups["ms"].Success ? (int)Math.Round((1000 * double.Parse(m.Groups["ms"].Value))) : 0;

                    TimeSpan ofs = TimeSpan.Zero;
                    if (m.Groups["ofs"].Success)
                    {
                        if (isRfc822)
                            ofs = ParseRfc822Offset(m.Groups["ofs"].Value);
                        else
                            ofs = ParseW3COffset(m.Groups["ofs"].Value);
                    }
                    // datetime is stored in UTC
                    return new W3CDateTime(new DateTime(year, month, day, hour, min, sec, ms) - ofs, ofs);
                }
                catch (Exception ex)
                {
                    throw new FormatException("String is not a valid date time stamp.", ex);
                }
            }


            // Format is "R" (RFC822) or "W" (W3C)
            public string ToString(string format)
            {
                switch (format)
                {
                    case "R":
                        return (dtime + ofs).ToString("ddd, dd MMM yyyy HH:mm:ss ") +
                          FormatOffset(ofs, "");
                    case "W":
                        return (dtime + ofs).ToString("yyyy-MM-ddTHH:mm:ss") +
                          FormatOffset(ofs, ":");
                    default:
                        throw new ArgumentException("Unrecognized date format requested.");
                }
            }

            private static string FormatOffset(TimeSpan ofs, string separator)
            {
                string s = string.Empty;
                if (ofs >= TimeSpan.Zero)
                    s = "+";
                return s + ofs.Hours.ToString("00") + separator + ofs.Minutes.ToString("00");
            }
        }

        static public W3CDateTime Parse(string s)
        {
            const string Rfc822DateFormat =
              @"^((Mon|Tue|Wed|Thu|Fri|Sat|Sun), *)?(?<day>\d\d?) +" +
              @"(?<month>Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) +" +
              @"(?<year>\d\d(\d\d)?) +" +
              @"(?<hour>\d\d):(?<min>\d\d)(:(?<sec>\d\d))? +" +
              @"(?<ofs>([+\-]?\d\d\d\d)|UT|GMT|EST|EDT|CST|CDT|MST|MDT|PST|PDT)$";
            const string W3CDateFormat =
              @"^(?<year>\d\d\d\d)" +
              @"(-(?<month>\d\d)(-(?<day>\d\d)(T(?<hour>\d\d):(?<min>\d\d)(:(?<sec>\d\d)(?<ms>\.\d+)?)?" +
              @"(?<ofs>(Z|[+\-]\d\d:\d\d)))?)?)?$";

            string combinedFormat = string.Format(
              @"(?<rfc822>{0})|(?<w3c>{1})", Rfc822DateFormat, W3CDateFormat);

            // try to parse it
            Regex reDate = new Regex(combinedFormat);
            Match m = reDate.Match(s);
            if (!m.Success)
            {
                // Didn't match either expression. Throw an exception.
                throw new FormatException("String is not a valid date time stamp.");
            }
            try
            {
                bool isRfc822 = m.Groups["rfc822"].Success;
                int year = int.Parse(m.Groups["year"].Value);
                // handle 2-digit and 3-digit years
                if (year < 1000)
                {
                    if (year < 50) year = year + 2000; else year = year + 1999;
                }

                int month;
                if (isRfc822)
                    month = ParseRfc822Month(m.Groups["month"].Value);
                else
                    month = (m.Groups["month"].Success) ? int.Parse(m.Groups["month"].Value) : 1;

                int day = m.Groups["day"].Success ? int.Parse(m.Groups["day"].Value) : 1;
                int hour = m.Groups["hour"].Success ? int.Parse(m.Groups["hour"].Value) : 0;
                int min = m.Groups["min"].Success ? int.Parse(m.Groups["min"].Value) : 0;
                int sec = m.Groups["sec"].Success ? int.Parse(m.Groups["sec"].Value) : 0;
                int ms = m.Groups["ms"].Success ? (int)Math.Round((1000 * double.Parse(m.Groups["ms"].Value))) : 0;

                TimeSpan ofs = TimeSpan.Zero;
                if (m.Groups["ofs"].Success)
                {
                    if (isRfc822)
                        ofs = ParseRfc822Offset(m.Groups["ofs"].Value);
                    else
                        ofs = ParseW3COffset(m.Groups["ofs"].Value);
                }
                // datetime is stored in UTC
                return new W3CDateTime(new DateTime(year, month, day, hour, min, sec, ms) - ofs, ofs);
            }
            catch (Exception ex)
            {
                throw new FormatException("String is not a valid date time stamp.", ex);
            }
        }

        private static readonly string[] MonthNames = new string[]
            {"Jan", "Feb", "Mar", "Apr", "May", "Jun", 
             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

        private static int ParseRfc822Month(string monthName)
        {
            for (int i = 0; i < 12; i++)
            {
                if (monthName == MonthNames[i])
                {
                    return i + 1;
                }
            }
            throw new ApplicationException("Invalid month name");
        }

        private static TimeSpan ParseRfc822Offset(string s)
        {
            if (s == string.Empty)
                return TimeSpan.Zero;
            int hours = 0;
            switch (s)
            {
                case "UT":
                case "GMT":
                    break;
                case "EDT": hours = -4; break;
                case "EST":
                case "CDT": hours = -5; break;
                case "CST":
                case "MDT": hours = -6; break;
                case "MST":
                case "PDT": hours = -7; break;
                case "PST": hours = -8; break;
                default:
                    if (s[0] == '+')
                    {
                        string sfmt = s.Substring(1, 2) + ":" + s.Substring(3, 2);
                        return TimeSpan.Parse(sfmt);
                    }
                    else
                        return TimeSpan.Parse(s.Insert(s.Length - 2, ":"));
            }
            return TimeSpan.FromHours(hours);
        }

        private static TimeSpan ParseW3COffset(string s)
        {
            if (s == string.Empty || s == "Z")
                return TimeSpan.Zero;
            else
            {
                if (s[0] == '+')
                    return TimeSpan.Parse(s.Substring(1));
                else
                    return TimeSpan.Parse(s);
            }
        }

        #endregion

        private bool loadItem(
            XmlNode item,
            List<MessageItem> messageLoadBuffer,
            FeedDescription feed)
        {
            debugOutput.PutMessageLine("   Item : " + item["title"].FirstChild.Value);
            string title = item["title"].FirstChild.Value;
            XmlNode descriptionItem = item["description"];

            if (descriptionItem == null)
            {
                debugOutput.PutMessageLine(" *** No description for " + title);
                return false;
            }

            if (descriptionItem.ChildNodes.Count==0)
            {
                debugOutput.PutMessageLine(" *** Empty description for " + title);
                return false;
            }

            string description = item["description"].FirstChild.Value;
            string pubDate = item["pubDate"].FirstChild.Value;

            W3CDateTime pubDateW3 = W3CDateTime.Parse(pubDate);

            if (DateTime.Now.Subtract(pubDateW3.DateTime) < feed.ItemAge)
            {
                messageLoadBuffer.Add(
                    new MessageItem(title, description + " [" + feed.Name + "]", pubDate, feed, messageLoadBuffer.Count + 1));
                return true;
            }
            return false;
        }

        private void loadChannel(
            XmlNode channel,
            List<MessageItem> messageLoadBuffer,
            FeedDescription feed,
            int maxNoOfMessages)
        {
            int messageCount = 0;

            debugOutput.PutMessageLine("Channel : " + channel["title"].FirstChild.Value);
            foreach (XmlNode item in channel.ChildNodes)
            {
                if (!item.Name.Equals("item"))
                {
                    continue;
                }
                if (loadItem(item, messageLoadBuffer, feed))
                {
                    if (++messageCount == maxNoOfMessages)
                    {
                        return;
                    }
                }
            }
        }

        private void loadMessages(
            XmlDocument doc,
            List<MessageItem> messageLoadBuffer,
            FeedDescription feed,
            int maxNoOfMessages)
        {
            XmlElement rss = doc["rss"];

            if (rss == null)
            {
                return;
            }

            // iterate through the channels
            foreach (XmlNode channel in rss.ChildNodes)
            {
                if (!channel.Name.Equals("channel"))
                {
                    continue;
                }
                loadChannel(channel, messageLoadBuffer, feed, maxNoOfMessages);
            }
        }

        private int loadMessages()
        {

            messageLoadBuffer.Clear();

            debugOutput.PutMessageLine("Message Reload at " + System.DateTime.Now.ToLongTimeString());

            foreach (FeedDescription feed in messageFeeds)
            {
                if (feed.URL.Length == 0)
                {
                    continue;
                }
                XmlDocument doc;

                doc = getRSSDoc(feed.URL, feed.Username, feed.Password, feed.Domain);

                if (doc == null)
                {
                    debugOutput.PutMessageLine("No messages document at " + feed.URL);
                }
                else
                {
                    loadMessages(doc, messageLoadBuffer, feed, feed.MaxNoOfMessages);
                }
            }

            return messageLoadBuffer.Count;
        }

        #endregion

        #region Message Update Threading

        private Thread updateThread;

        public void RequestUpdate()
        {
            updateThread = new Thread(doCheckUpdate);
            updateThread.Start();
        }

        public void ShutDown()
        {
        }

        private void noMessages()
        {
            messageLoadBuffer.Add(
                new MessageItem(
                    "No Messages Available",
                    "<p>No messages are available at the moment</P>" +
                    DateTime.Now.ToLongDateString() + " " + DateTime.Now.ToShortTimeString(),
                    DateTime.Now.ToLongTimeString(),
                    FeedDescription.NullFeed,
                    1)
            );
        }

        private void doCheckUpdate()
        {
            if (loadMessages() == 0)
            {
                noMessages();
            }
            if (Updated != null)
            {
                Updated(messageLoadBuffer);
            }
        }

        #endregion

        private IDoDebug debugOutput;
        private FeedDescription[] messageFeeds;

        public Messages(FeedDescription[] inMessageFeeds, IDoDebug debug)
        {
            debugOutput = debug;
            messageFeeds = inMessageFeeds;
            noMessages();
        }

        public override string ToString()
        {
            StringBuilder reply = new StringBuilder();

            reply.AppendLine("Message Reader");

            foreach (FeedDescription feed in messageFeeds)
            {
                reply.AppendLine(feed.ToString());
            }

            reply.AppendLine("Number of messages : " + messageLoadBuffer.Count);

            foreach (MessageItem message in messageLoadBuffer)
            {
                reply.AppendLine(message.ToString());
                reply.AppendLine();
            }

            return reply.ToString();
        }
    }
}
