//#define FULL_SCREEN
#define FIXED_MESSAGES

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SillyText
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class MessageSystemGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        GamerServicesComponent services;
        SpriteBatch spriteBatch;

        IMessageUpdate messageReader;
        FeedDescription[] messageSources = new FeedDescription[] {
//            new FeedDescription ( @"http://www.robmiles.com/magic-message-rss-feed/rss.xml","","",""),
            //new FeedDescription ( 
            //    @"http://intra.net.dcs.hull.ac.uk/sites/home/_layouts/listfeed.aspx?List=%7B3289D18E%2D8154%2D435B%2D8BED%2DFF065D74A033%7D",
            //    "DCS",
            //    new TimeSpan(30,0,0,0,0),
            //    20,
            //    "cssrsm","fiatpunto","net"),
            new FeedDescription ( 
                @"http://verysillygames.com/silly-message-feed/rss.xml",
                "",
                new TimeSpan(100,0,0,0,0),
                100)
            //new FeedDescription ( 
            //    @"http://newsrss.bbc.co.uk/rss/newsonline_uk_edition/uk/rss.xml",
            //    "BBC",
            //    new TimeSpan(0,10,0,0),
            //    10)

        };

        ConsoleDebug debug = new ConsoleDebug();

        private List<MessageItem> rssMessages;
        private List<DisplayMessage> messages = new List<DisplayMessage>();

        private void initializeMessages()
        {
#if FIXED_MESSAGES
            messageReader = new JustMessages();

#else

            messageReader = new Messages(messageSources, debug);
#endif
            messageReader.Updated += messageUpdate;
            messageReader.RequestUpdate();
        }

        SpriteFont messageFont;

        float headingNWidth = 40;
        float nWidth = 40;

        Color[] colors = new Color[] { Color.Red };
        Color[] sides = new Color[] { Color.Black };
        Color[] shadows = new Color[] { new Color(0, 0, 0, 110) };

        private class DisplayMessage
        {
            ISillyText currentHeadingString = null;
            ISillyText currentMessageString = null;
            MessageItem currentMessageItem = null;

            public DisplayMessage(ISillyText heading, ISillyText message, MessageItem item)
            {
                currentHeadingString = heading;
                currentMessageString = message;
                currentMessageItem = item;
            }

            public void Draw(SpriteBatch spriteBatch)
            {
                currentHeadingString.Draw(spriteBatch);
                currentMessageString.Draw(spriteBatch);
            }

            public void Update()
            {
                currentHeadingString.Update();
                currentMessageString.Update();
            }

            public bool FinishedMoving()
            {
                if (currentHeadingString.IsMoving() ||
                    currentMessageString.IsMoving())
                {
                    return false;
                }
                return true;
            }

            public void FlyIn()
            {
                currentHeadingString.FlyInState = SillyWord.FlyInStates.flyingIn;
                currentMessageString.FlyInState = SillyWord.FlyInStates.flyingIn;
            }

            public void FlyOut()
            {
                currentHeadingString.FlyInState = SillyWord.FlyInStates.flyingOut;
                currentMessageString.FlyInState = SillyWord.FlyInStates.flyingOut;
            }

            public void StartScroll()
            {
                currentHeadingString.OffsetX = 0;
                currentHeadingString.OffsetY = 0;
                currentMessageString.OffsetX = 0;
                currentMessageString.OffsetY = 0;
            }

            public int MessageHeight
            {
                get
                {
                    return (int)(currentMessageString.Height + 0.5f);
                }
            }

            public bool NeedsScroll(float maxY)
            {
                if ((currentMessageString.Y +
                    currentMessageString.Height +
                    currentMessageString.OffsetY) < maxY)
                {
                    return false;
                }
                return true;
            }

            public bool UpdateYScroll(float distance, float maxY)
            {
                if (!NeedsScroll(maxY))
                {
                    return false;
                }

                currentHeadingString.OffsetY -= distance;
                currentMessageString.OffsetY -= distance;

                return true;
            }
        }

        private int messageNo = 0;
        private bool newMessages = false;

        private void messageUpdate(List<MessageItem> newMessageItems)
        {
            rssMessages = newMessageItems;
            newMessages = true;
        }

        Random colorRand = new Random(1);

        Color[] messageColors = new Color[] { 
                Color.Red, Color.Blue, Color.Green, Color.Orange, Color.Pink, 
                Color.LightBlue, Color.OrangeRed, Color.DarkBlue, Color.Yellow,
                Color.Magenta, Color.Purple };


        private void loadMessages()
        {
            messages.Clear();

            foreach (MessageItem item in rssMessages)
            {
                int headingColor = colorRand.Next(messageColors.Length);
                int messageColor;
                do
                {
                    messageColor = colorRand.Next(messageColors.Length);
                }
                while (headingColor == messageColor);

                colors[0] = messageColors[headingColor];
                ISillyText currentHeadingString = new SillyHtmlLine(
                    item.Title,
                    new Vector2(minDisplayX, minDisplayY),
                    colors[0],
                    Color.Black,
                    Color.Red,
                    new Color(0, 0, 0, 110),
                    messageFont,
                    headingNWidth,
                    maxDisplayX);

                currentHeadingString.FlyInState = SillyWord.FlyInStates.flyingIn;

                ISillyText currentMessageString = new SillyHtmlLine(
                    item.BodyString,
                    new Vector2(minDisplayX, currentHeadingString.Height + 20),
                    messageColors[messageColor],
                    Color.Black,
                    Color.Red,
                    new Color(0, 0, 0, 110),
                    messageFont,
                    nWidth,
                    maxDisplayX);

                currentMessageString.FlyInState = SillyWord.FlyInStates.flyingIn;

                messages.Add(
                    new DisplayMessage(currentHeadingString, currentMessageString, item));
            }
        }

        /// <summary>
        /// Counts the number of ticks during display
        /// </summary>
        int messageTimer = 0;

        /// <summary>
        /// Limit of time out value
        /// </summary>
        int messageDisplayTime = 100;

        float scrollProportion = 0.001f;
        float scrollstep;

        Slider slide;

        public MessageSystemGame()
        {
            graphics = new GraphicsDeviceManager(this);

            //this.graphics.PreferredBackBufferWidth = 1280;
            //this.graphics.PreferredBackBufferHeight = 720;

            //this.graphics.IsFullScreen = true;

            Content.RootDirectory = "Content";

            if (wantFullScreen)
            {
                graphics.PreparingDeviceSettings +=
                            new EventHandler<PreparingDeviceSettingsEventArgs>(
                            graphics_PreparingDeviceSettings);
            }

            // Needed for using the Guide.
            services = new GamerServicesComponent(this);

            Components.Add(services);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            setupScreenSizeInfo();
            setupGuide();
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            messageFont = Content.Load<SpriteFont>("MessageFont");

            initializeMessages();

            slide = new Slider(graphics);

            slide.LoadContent(Content,"MoodCircles");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        enum DisplayState
        {
            starting,
            fadingUp,
            holdingMessage,
            scrollingMessage,
            displayingMessage,
            fadingDown
        };

        DisplayState state = DisplayState.starting;

        private void MessageCallback(IAsyncResult result)
        {
            Console.WriteLine(Guide.EndShowKeyboardInput(result));
        }

        private KeyboardState oldKeyboardState;

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            GamePadState pad1 = GamePad.GetState(PlayerIndex.One);
            KeyboardState keyboard = Keyboard.GetState();

            // Allows the game to exit
            if (pad1.Buttons.Back == ButtonState.Pressed ||
                keyboard.IsKeyDown(Keys.Escape))
            {
                messageReader.ShutDown();
                this.Exit();
            }

            //if (keyboard.IsKeyDown(Keys.Enter) && oldKeyboardState.IsKeyUp(Keys.Enter))
            //{
            //    Guide.BeginShowKeyboardInput(PlayerIndex.One, "Display resolution", "Setting Display Resultion", "1024x768", MessageCallback, null);
            //}

            oldKeyboardState = keyboard;

            slide.Update(gameTime);

            switch (state)
            {
                case DisplayState.starting:
                    if (newMessages)
                    {
                        loadMessages();
                        messageNo = 0;
                        newMessages = false;
                        messageReader.RequestUpdate();
                        state = DisplayState.fadingUp;
                        break;
                    }
                    else
                    {
                        return;
                    }

                case DisplayState.fadingUp:
                    if (messages[messageNo].FinishedMoving())
                    {

                        messageTimer = 0;

                        if (messages[messageNo].NeedsScroll(maxDisplayY))
                        {
                            state = DisplayState.holdingMessage;
                        }
                        else
                        {
                            state = DisplayState.displayingMessage;
                        }
                    }
                    break;

                case DisplayState.holdingMessage:
                    if (++messageTimer == messageDisplayTime)
                    {
                        messages[messageNo].StartScroll();
                        state = DisplayState.scrollingMessage;
                    }
                    break;

                case DisplayState.scrollingMessage:
                    if (!messages[messageNo].UpdateYScroll(scrollstep, maxDisplayY))
                    {
                        messageTimer = 0;
                        state = DisplayState.displayingMessage;
                    }
                    break;

                case DisplayState.displayingMessage:

                    if (++messageTimer == messageDisplayTime)
                    {
                        messages[messageNo].FlyOut();
                        state = DisplayState.fadingDown;
                    }
                    break;

                case DisplayState.fadingDown:
                    if (messages[messageNo].FinishedMoving())
                    {
                        messageNo++;
                        if (messageNo == messages.Count)
                        {
                            if (newMessages)
                            {
                                loadMessages();
                                newMessages = false;
                                messageReader.RequestUpdate();
                            }
                            messageNo = 0;
                        }
                        messages[messageNo].FlyIn();
                        state = DisplayState.fadingUp;
                    }
                    break;

            }

            messages[messageNo].Update();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.White);

            slide.Draw(spriteBatch, graphics);

            if (state != DisplayState.starting)
            {
                spriteBatch.Begin();

                if (messages.Count > 0)
                {
                    messages[messageNo].Draw(spriteBatch);
                }
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }

        #region Display dimension values

        // Display settings
        int rawDisplayWidth;
        int rawDisplayHeight;
        int minDisplayX;
        int maxDisplayX;
        int minDisplayY;
        int maxDisplayY;
        int displayWidth;
        int displayHeight;

        float overScanPercentage = 10.0f;
        private Rectangle screenRectangle;

        /// <summary>
        /// Calculates percentages
        /// </summary>
        /// <param name="percentage">the percentage to be calculated</param>
        /// <param name="inputValue">the value to be worked on</param>
        /// <returns>the resulting value</returns>
        float getPercentage(float percentage, float inputValue)
        {
            return (inputValue * percentage) / 100;
        }

        //private int desiredWidth = 1440;
        //private int desiredHeight = 900;

        private int desiredWidth = 1280;
        private int desiredHeight = 1024;

        private bool wantFullScreen = false;

        /// <summary>
        /// Modifies the display mode for the graphics device when it is reset or recreated.
        /// </summary>
        void graphics_PreparingDeviceSettings(object sender, PreparingDeviceSettingsEventArgs e)
        {
            foreach (Microsoft.Xna.Framework.Graphics.DisplayMode displayMode in GraphicsAdapter.DefaultAdapter.SupportedDisplayModes)
            {
                if (displayMode.Width == desiredWidth && displayMode.Height == desiredHeight)
                {
                    e.GraphicsDeviceInformation.PresentationParameters.BackBufferFormat = displayMode.Format;
                    e.GraphicsDeviceInformation.PresentationParameters.BackBufferHeight = displayMode.Height;
                    e.GraphicsDeviceInformation.PresentationParameters.BackBufferWidth = displayMode.Width;
                    e.GraphicsDeviceInformation.PresentationParameters.FullScreenRefreshRateInHz = displayMode.RefreshRate;
                    e.GraphicsDeviceInformation.PresentationParameters.IsFullScreen = true;
                }
            }
        }

        private void setupScreenSizeInfo()
        {
            rawDisplayWidth = graphics.GraphicsDevice.Viewport.Width;
            rawDisplayHeight = graphics.GraphicsDevice.Viewport.Height;

            float xOverscanMargin = getPercentage(overScanPercentage, rawDisplayWidth) / 2.0f;
            float yOverscanMargin = getPercentage(overScanPercentage, rawDisplayHeight) / 2.0f;

            minDisplayX = (int)(xOverscanMargin + 0.5f);
            minDisplayY = (int)(yOverscanMargin + 0.5f);

            maxDisplayX = rawDisplayWidth - minDisplayX;
            maxDisplayY = rawDisplayHeight - minDisplayY;

            displayWidth = maxDisplayX - minDisplayX;
            displayHeight = maxDisplayY - minDisplayY;
            screenRectangle = new Rectangle(minDisplayX, minDisplayY, displayWidth, displayHeight);

            nWidth = displayWidth / 30;
            scrollstep = displayHeight * scrollProportion;
        }

        #endregion

        private void setupGuide()
        {
            Guide.IsScreenSaverEnabled = false;
        }
    }

}
