#define FULL_SCREEN

using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SillyText
{
    public class PictureLayer
    {
        public Rectangle rect;
        public Texture2D texture;

        public int lowX;
        public int lowY;
        public int highX;
        public int highY;
        public bool xIncreasing;
        public bool yIncreasing;

        public void Update()
        {
            if (xIncreasing)
            {
                rect.X++;
                if (rect.X == highX)
                {
                    xIncreasing = false;
                }
            }
            else
            {
                rect.X--;
                if (rect.X == lowX)
                {
                    xIncreasing = true;
                }
            }

            if (yIncreasing)
            {
                rect.Y++;
                if (rect.Y == highY)
                {
                    yIncreasing = false;
                }
            }
            else
            {
                rect.Y--;
                if (rect.Y == lowY)
                {
                    yIncreasing = true;
                }
            }
        }

        public void Draw(SpriteBatch spriteBatch, Color drawColor)
        {
            spriteBatch.Draw(texture, rect, drawColor);
        }

        public PictureLayer(
            Rectangle inRect,
            Texture2D inTexture,
            int inLowX, int inHighX, int inLowY, int inHighY,
            bool inXIncreasing, bool inYIncreasing)
        {
            rect = inRect;
            texture = inTexture;
            lowX = inLowX;
            highX = inHighX;
            lowY = inLowY;
            highY = inHighY;
            xIncreasing = inXIncreasing;
            yIncreasing = inYIncreasing;
        }
    }

    class Slider
    {
        Texture2D gameTexture;
        PictureLayer[] layers = new PictureLayer[5];
        Vector2 timeVector;
        byte background = 255;
        byte transparency = 60;
        byte redIntensity = 240;
        byte greenIntensity = 240;
        byte blueIntensity = 240;
        public float rot;
        public float layerRot = 0;
        public float rotSpeed = 0;

        int screenWidth;
        int screenHeight;

        Random rand;


        public Slider(GraphicsDeviceManager graphics)
        {
#if FULL_SCREEN
            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 768;
            graphics.PreferMultiSampling = false;
            graphics.IsFullScreen = true;
#endif
            rand = new Random(1);

            screenWidth = graphics.GraphicsDevice.Viewport.Width;
            screenHeight = graphics.GraphicsDevice.Viewport.Height;
        }

        public void LoadContent(ContentManager Content, string assetName)
        {
            gameTexture = Content.Load<Texture2D>(assetName);
            for (int i = 0; i < layers.Length; i++)
            {
                int factor = i + 1;
                Rectangle rect = new Rectangle(
                    rand.Next(-screenWidth * factor, 0),
                    rand.Next(-screenHeight * factor, 0),
                    screenWidth * factor * 2,
                    screenHeight * factor * 2);

                layers[i] = new PictureLayer(rect,
                    gameTexture,
                    -screenWidth * factor, 0,
                    -screenHeight * factor, 0,
                    rand.Next(2) == 0,
                    rand.Next(2) == 0);
            }
        }

        public void Update(GameTime gameTime)
        {
            GamePadState pad1 = GamePad.GetState(PlayerIndex.One);

            if (pad1.Buttons.Start == ButtonState.Pressed)
            {
                background = 0;
                transparency = 180;
                redIntensity = 240;
                greenIntensity = 240;
                blueIntensity = 240;
                layerRot = 0;
                rotSpeed = 0;
                rot = 0;
                timeVector.X = 0;
                timeVector.Y = 0;
            }

            // Adjust the time position
            timeVector.X += (pad1.ThumbSticks.Left.X * 5);
            timeVector.Y -= (pad1.ThumbSticks.Left.Y * 5);

            // Adjust the transparency of the colors
            if (pad1.DPad.Up == ButtonState.Pressed) transparency++;
            if (pad1.DPad.Down == ButtonState.Pressed) transparency--;
            if (pad1.DPad.Left == ButtonState.Pressed) background--;
            if (pad1.DPad.Right == ButtonState.Pressed) background++;

            // Update the colors
            if (pad1.Buttons.B == ButtonState.Pressed) redIntensity++;
            if (pad1.Buttons.X == ButtonState.Pressed) blueIntensity++;
            if (pad1.Buttons.A == ButtonState.Pressed) greenIntensity++;

            rotSpeed += (pad1.ThumbSticks.Right.X * 0.0001f);
            layerRot += (pad1.ThumbSticks.Right.Y * 0.0001f);

            if (pad1.Buttons.Y == ButtonState.Pressed)
            {
                redIntensity++;
                greenIntensity++;
            }

            // Update all the layers
            for (int i = 0; i < layers.Length; i++)
            {
                layers[i].Update();
            }

            rot = rot + rotSpeed;

        }

        public void Draw(SpriteBatch spriteBatch, GraphicsDeviceManager graphics)
        {
            Color back = new Color(background, background, background);
            graphics.GraphicsDevice.Clear(back);
            Color drawColor =
                new Color(redIntensity, greenIntensity, blueIntensity, transparency);
            for (int i = 0; i < layers.Length; i++)
            {
                PictureLayer p = layers[i];
                spriteBatch.Begin(
                    SpriteBlendMode.AlphaBlend,
                    SpriteSortMode.BackToFront,
                    SaveStateMode.None,
                    Matrix.CreateRotationZ(rot + i * layerRot));
                p.Draw(spriteBatch, drawColor);
                spriteBatch.End();
            }
        }
    }
}
